import jwt
import time
from pyjwt512.Es512KeysManger import Es512KeysManger
from pyjwt512.Exceptions import CreateTokenException


class CreateJwtToken:
    def __init__(self, cert_dir: str, payload: dict) -> None:
        """
        Initialize the JWT Token creator with a directory for certificates and a payload.
        The payload must include "iss" (issuer), "aud" (audience), and "uid" (user identifier).

        Params:
            cert_dir: Directory with keys.
            payload: Payload with required values.
        """
        self.__check_payload(payload)
        self.__payload = payload

        self.__cert_dir = cert_dir
        self.__token: str = None

    def get_token(self) -> str | None:
        """
        Returns the generated token.

        Returns:
            str: The JWT token, or None if not yet created.
        """
        return self.__token

    def create(self) -> bool:
        """
        Create a JWT Token. ES512 keys must exist or be created before calling this method.

        Returns:
            bool: True if the token was successfully created.

        Raises:
            CreateTokenException: If there's an error in token creation.
        """
        # load last keys
        es512 = Es512KeysManger()
        if not es512.load_keys(cert_dir=self.__cert_dir, public_only=False):
            raise CreateTokenException("Failed to load ES512 keys for token creation.")

        # add dactual iat
        self.__payload["iat"] = int(time.time())  # actual unix timestamp
        # set kid
        headers = {"kid": es512.get_root_filename()}

        self.__token = jwt.encode(
            self.__payload, es512.get_priv_cert(), algorithm="ES512", headers=headers
        )

        return True

    def __check_payload(self, payload: dict) -> None | CreateTokenException:
        """
        Checks if the payload contains all required keys with valid types.

        Params:
            payload: The payload to check.

        Raises:
            CreateTokenException: If required payload arguments are missing or invalid.
        """
        required_keys = {
            "iss": str,  # Issuer expected to be a string
            "aud": str,  # Audience expected to be a string
            "uid": int,  # User ID expected to be an integer
        }

        for key, expected_type in required_keys.items():
            if key not in payload:
                raise CreateTokenException(
                    f"Missing required payload argument: '{key}'."
                )
            if not isinstance(payload[key], expected_type):
                raise CreateTokenException(
                    f"Incorrect type for payload argument '{key}': Expected {expected_type.__name__}, got {type(payload[key]).__name__}."
                )



#!/usr/bin/env python3
import os
import fire
from pyjwt512.Exceptions import CreateTokenException, InvalidTokenException
from pyjwt512.CreateJwtToken import CreateJwtToken
from pyjwt512.Es512KeysManger import Es512KeysManger
from pyjwt512.VerifyJwtToken import VerifyJwtToken


class jwt_tokens:
    """
    Generate JWT, Create new ECDSA SHA-512 keys or validate a token with key.
    """

    def token(self, dir: str, iss: str, aud: str, uid: int) -> None:
        """
        Create new JWT token.

        Required arguments:
            --dir=<path>: str
            --iss=<issuer>: str
            --aud=<audince>: str
            --uid=<id>: int
        """
        if os.path.exists(dir) and uid and iss and aud:
            payload = {
                "iss": iss,
                "aud": aud,
                "uid": uid,
            }
            try:
                create_token = CreateJwtToken(cert_dir=dir, payload=payload)
                if create_token.create():
                    print("Token: ", create_token.get_token())
            except CreateTokenException as e:
                print(f"CreateTokenException: {e}")
        else:
            print(f"Args error. Check: --dir={dir} --iss={iss} --aud={aud} --uid={uid}")

    def keys(self, dir: str) -> None:
        """
        Create new PEM KEYS and save it in DIR

        Required argument:
            --dir=<path>
        """
        if os.path.exists(dir):
            es512 = Es512KeysManger()
            es512.generate_new_keys()
            if not es512.save_new_keys(cert_dir=dir):
                print(f"ERROR: Keys not generated.")
            k_priv = os.path.join(dir, f"{es512.get_root_filename()}.pem")
            k_pub = os.path.join(dir, f"{es512.get_root_filename()}-public.pem")
            print(f"New keys has been saved in {k_pub} and {k_priv} files.")
        else:
            print(f"Args error. Check: --dir={dir}")

    def check(self, token: str, dir: str, aud: str) -> None:
        """
        Check validity of a token

        Required arguments:
            --dir=<path>: str
            --aud=<audience>: str
            --token=<jwt token>: str
        """
        if os.path.exists(dir) and token and aud:
            jwt_token = VerifyJwtToken()
            try:
                if jwt_token.validate(token=token, audience=aud, cert_dir=dir):
                    print(f"Token is valid.")
                    print(jwt_token)
            except InvalidTokenException as e:
                print(f"InvalidTokenException: {e}")
        else:
            print(f"Args error. Check: --dir={dir} --token={token[:8]}...  --aud={aud}")

def run():
    fire.Fire(jwt_tokens)

if __name__ == "__main__":
    run()

#if __name__ == "__main__":
#    fire.Fire(jwt_tokens)



import os
import jwt
from pyjwt512.Exceptions import InvalidTokenException


class VerifyJwtToken:
    """
    A class for verifying JSON Web Tokens (JWT) using the ES384 algorithm.

    Attributes:
        iss (str): Issuer of the JWT.
        aud (str): Audience of the JWT.
        iat (str): Issued At time of the JWT.
        uid (int): User ID in the JWT.
        kid (str): Key ID for the public key used in verification.

    Methods:
        validate(token, audience, cert_dir): Validates the JWT. Returns True if valid, otherwise raises InvalidTokenExeption.
        __str__(): Returns a string representation of the object.
    """

    def __init__(self) -> None:
        self.__iss: str = None
        self.__aud: str = None
        self.__iat: str = None
        self.__uid: int = None
        self.__kid: str = None

    def get_iss(self) -> str:
        """Get iss"""
        return self.__iss

    def get_aud(self) -> str:
        """Get sud"""
        return self.__aud

    def get_iat(self) -> str:
        """Get iat"""
        return self.__iat

    def get_uid(self) -> int:
        """Get uid"""
        return self.__uid

    def get_kid(self) -> str:
        """Get kid"""
        return self.__kid

    def validate(
        self, token: str, audience: str, cert_dir: str
    ) -> bool | InvalidTokenException:
        """
        Validates a JWT token using the ES512 algorithm.

        This function attempts to verify the given JWT token by checking its signature
        with the public key specified by the 'kid' (Key ID) field in the token. The
        public key is retrieved from the provided certificate directory. The function
        also verifies that the token's audience matches the provided audience.

        Params:

            token (str): The JWT token to be validated.

            audience (str): The expected audience value that the token should contain.
            
            cert_dir (str): The directory where the public key PEM file is located.
            The public key file is expected to be named as '<kid>-public.pem',
            where '<kid>' is the Key ID obtained from the token.

        Returns:
            bool: True if the token is successfully validated, False otherwise.

        Raises:
            InvalidTokenException: If the token cannot be decoded, if the Key ID ('kid')
                                   is missing, if the public key file cannot be found or read,
                                   or if the token's signature, audience, issuer, etc., are
                                   invalid or do not match the expected values.

        Example::

            try:
                verifier = VerifyJwtToken()
                is_valid = verifier.validate(token="your.jwt.token", audience="yourAudience", cert_dir="/path/to/cert/dir")
                if is_valid:
                    print('Token is valid.')
                else:
                    print('Token is invalid.')
            except InvalidTokenException as e:
                print(f'Token validation failed: {e}')
        """

        try:
            token_data = jwt.get_unverified_header(token)
            self.__kid = token_data["kid"]
        except jwt.exceptions.DecodeError as e:
            raise InvalidTokenException(e)

        # Load public key
        try:
            public_key_path = os.path.join(cert_dir, f"{self.__kid}-public.pem")
            with open(public_key_path, "r") as f:
                public_key = f.read()
        except IOError as e:
            raise InvalidTokenException(str(e))

        try:
            # jwt token validation
            decoded_jwt = jwt.decode(
                token, public_key, audience=audience, algorithms=["ES512"]
            )
            self.__iss = decoded_jwt["iss"]
            self.__aud = decoded_jwt["aud"]
            self.__iat = decoded_jwt["iat"]
            self.__uid = decoded_jwt["uid"]
        except jwt.InvalidTokenError as e:
            raise InvalidTokenException(str(e))
        except Exception as e:
            raise InvalidTokenException(str(e))

        return True

    def __str__(self) -> str:
        return f"iss : {self.__iss}, aud : {self.__aud}, iat : {self.__iat}, uid : {self.__uid}, kid : {self.__kid}"



import os
import re
import uuid
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ec


class Es512KeysManger:
    """
    Handles generation, loading, and saving of ES512 private and public keys.

    Methods:
        get_root_filename(self): Returns the root file name for keys.
        get_pub_cert(self): Returns the public key in PEM format.
        get_priv_cert(self): Returns the private key in PEM format.
        generate_new_keys(self): Generates new ES512 private and public keys.
        save_new_keys(self, cert_dir): Saves the generated keys to the specified directory.
        load_keys(self, cert_dir, public_only=True, filename=""): Loads the latest keys from the specified directory or specified keys in filename="".
    """

    def __init__(self) -> None:
        # primary file name root string
        self.__filename = uuid.uuid4().hex

        # __pem_privkey and __pem_pubkey are generated by this class
        self.__generated = False

        # pem keys (generated or loaded)
        self.__pem_privkey: str = None
        self.__pem_pubkey: str = None

    def get_root_filename(self) -> str:
        # Return root of filename keys
        return str(self.__filename)

    def get_pub_cert(self) -> str | None:
        # Return public key
        return self.__pem_pubkey

    def get_priv_cert(self) -> str | None:
        # Return private key
        return self.__pem_privkey

    def generate_new_keys(self) -> None:
        """
        Generate new ES512 keys
        """
        # Generate a private key for use in the exchange.
        privkey = ec.generate_private_key(ec.SECP521R1(), default_backend())

        # Serialize private key to PEM format
        self.__pem_privkey = privkey.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

        # Generate a public key from the private key
        public_key = privkey.public_key()

        # Serialize public key to PEM format
        self.__pem_pubkey = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )

        self.__generated = True

    def save_new_keys(self, cert_dir: str) -> bool:
        """
        Save generated keys to files

        Input:
            cert_dit: Direcory with keys
        """
        # protect save loaded certs
        if not self.__generated:
            return False

        try:
            priv_path = os.path.join(cert_dir, f"{self.__filename}.pem")
            pub_path = os.path.join(cert_dir, f"{self.__filename}-public.pem")

            # Save the public and private key to a files
            with open(priv_path, "wb") as f:
                f.write(self.__pem_privkey)

            with open(pub_path, "wb") as f:
                f.write(self.__pem_pubkey)

        except IOError as e:
            return False

        return True

    def load_keys(
        self, cert_dir: str, public_only: bool = True, filename: str = ""
    ) -> bool:
        """
        Load last keys to class

        Input:
            cert_dit: str Direcory with keys
            public_only: bool Set to False if you need load both keys
            filename: str If blank last created keys will be loaded
        """
        self.__generated = False
        try:
            # Ensure the directory exists
            if not os.path.isdir(cert_dir):
                return False

            if not filename:
                filename = self.__find_latest_key_filename(cert_dir)
                if not filename:
                    return False

            return self.__load_key_files(cert_dir, filename, public_only)

        except Exception:
            return False

    def __find_latest_key_filename(self, cert_dir: str) -> str | None:
        """
        Finds the latest key filename in the specified directory.

        Args:
            cert_dir (str): The directory to search for key files.

        Returns:
            str or None: The latest key filename or None if no keys are found.
        """
        try:
            pem_files = [f for f in os.listdir(cert_dir) if f.endswith(".pem")]
            pem_files.sort(
                key=lambda x: os.path.getmtime(os.path.join(cert_dir, x)), reverse=True
            )

            if pem_files:
                match = re.search(r"^([a-f0-9]{32})(-public)?\.pem$", pem_files[0])

                if match:
                    self.__filename = match.group(1)
                    return match.group(1)
                return ""
        except OSError:
            return None

    def __load_key_files(self, cert_dir: str, filename: str, public_only: bool) -> bool:
        """
        Loads key files from the specified directory.

        Args:
            cert_dir (str): The directory containing the key files.
            filename (str): The filename of the keys to load.
            public_only (bool): Whether to load only the public key.

        Returns:
            bool: True if keys are successfully loaded, False otherwise.

        Raises:
            IOError: If there is an issue reading the key files.
        """
        try:
            if not public_only:
                with open(os.path.join(cert_dir, f"{filename}.pem"), "r") as f:
                    self.__pem_privkey = f.read()

            with open(os.path.join(cert_dir, f"{filename}-public.pem"), "r") as f:
                self.__pem_pubkey = f.read()

            return True

        except IOError:
            # Log the IOError details here
            return False
